#/usr/bin/python3
'''
Created on 21. 6. 2014

@author: Filip Dobrovolny soutezici 61
Tento program umi vykreslit jedno patro
using python3
'''
import pygame, sys
from pygame.locals import *
from os import listdir
from os.path import isfile, join

'''
    @note: trida pro jednotlive kostky
    @param color: barva v RGB (RRR, GGG, BBB)
    @param width: sirka x
    @param height: sirka y
    @warning: width = self.sizex and height = self.sizey 
    @author: Filip Dobrovolny
'''
class lego(object):
    def __init__(self, color, width, height):
        self.color = color
        self.sizex = width
        self.sizey = height
    ''' 
        @note: rotates the cube to Y rotation for better building
        @author: Filip Dobrovolny
    '''
    def rotate(self): # this makes building easier
        if self.sizex > self.sizey:
            self.sizex, self.sizey = self.sizey, self.sizex

'''
    @note: Tower object which have data about size, type and blueprints
    @param x: x size
    @param y: y size
    @param type: type of tower
    @param plan: default None Blueprints of tower e.g. x = 2 y = 2 {1 : [[cube, cube], [cube2, cube2]], 2 : [[...],[...]]} 
    @author: Filip Dobrovolny  
 '''
class tower(object):
    def __init__(self, x, y, type, plan=None):
        self.x = x
        self.y = y
        self.type = type
        if plan == None:
            self.plan == {}
        else:
            self.plan = plan
        self.basePlan = genBasePlan(x, y)

'''
    @note: parse input parametrs
    @return: draw = 1 / plan = 0;; path ;; None / number of floor
    @author: Filip Dobrovolny
'''
def parseParams():
    arg = sys.argv
    if len(arg) == 1:
        print("Avalible arguments: ")
        print("{} - povinny parametr")
        print("[] - nepovinny parametr")
        print("plan {path} - Generovani stavebniho planu eg, /foo/bar bez posledniho lomitka!")
        print("draw {path} {patro} - vykresli věz")
        sys.exit()
    elif arg[1] == "draw":
        return 1, arg[2], arg[3]
    elif arg[1] == "plan":
        return 0, arg[2], None
    else:
        print("Invlaid command syntax")
        sys.exit()


''' 
    @note: Generating 2D field
    @return: e.g. x = 2 ; y = 2; return: [["", ""], ["", ""]]
    @author: Filip Dobrovolny 
'''
def genBasePlan(x, y):
    basePlan = []
    for i in range(0,x):
        temp = []
        for z in range(0, y):
            temp.append("")
        basePlan.append(temp)
    return basePlan


'''
    @note: decode colors from ARGBY to RGB
    @return: RGB tuple (RRR, GGG, BBB)
    @author: Filip Dobrovolny
'''
def decodeColor(color):
    colors = {"A" : (0, 255, 255), "R" : (255,0,0), "G" : (0,255,0), "B" : (0,0,255), "Y" : (255,255,0)}
    return colors[color]


'''
    @note: analyze one row of blueprint positioning
    @return: updated blueprints
    @author: Filip Dobrovolny
'''
def analyzeLegoPlan(row, plan, basePlan):
    patro = row[0]
    x = int(row[1])-1
    y = int(row[2])-1
    color = decodeColor(row[3])
    sizex = int(row[4][0])
    sizey = int(row[4][2])
    legoObj = lego(color, sizex, sizey)
    try:
        plan[patro]
    except KeyError:
        plan[patro] = basePlan
    plan[patro][x][y] = legoObj
    if sizex > 1:
        for i in range(1, sizex):
            plan[patro][x+i][y]
    if sizey > 1:
        for i in range(1, sizey):
            plan[patro][x][y+i]
    return plan
    

'''
    @note: Read and Remove from file whitelines and multiple spaces
    @param path: PAth of File 
    @return: Clear file 2D field 
    @author: Filip Dobrovolny

'''
def parseFile(path):
    file = open(path, "r") # nacteme soubor
    soubor = file.read() # precteme soubor
    file.close() # zavreme soubor
    while True: # zbavime se vicenasobnych mezer
        soubor = soubor.replace("  ", " ")
        if soubor.find("  ") == -1:
            break
    while True: # zbavime se vicenasobnych mezer
        soubor = soubor.replace("\n\n", "\n")
        if soubor.find("\n\n") == -1:
            break
    while True: # zbavime se vicenasobnych mezer
        soubor = soubor.replace("\n ", "\n")
        if soubor.find("\n ") == -1:
            break    
    soubor = soubor.split("\n")
    newSoubor = []
    for i in soubor:
        newSoubor.append(i.split(" "))
    for i in newSoubor:
        while True:
            try:
                i.remove("")
            except ValueError:
                break
    while True:
        try:
            newSoubor.remove([])
        except ValueError:
            break
    return newSoubor


''' 
    @note: Convert Blueprints to Tower object
    @param path: path to blueprint 
    @return: tower object
    @author: Filip Dobrovolny
'''
def parseBuildPlan(path):
    newSoubor = parseFile(path)
    x = int(newSoubor[0][0])
    y = int(newSoubor[0][1])
    type = newSoubor[0][2]
    plan = {}
    for i in range(1, len(newSoubor)-1):
        BasePlan = genBasePlan(x, y)
        plan = analyzeLegoPlan(newSoubor[i], plan, BasePlan)
    return tower(x, y, type, plan)
    

'''
    @note: Analyze one row of material list
    @param row: row from material list splitted to list
    @return: lego object and how many units of it are there
    @author: Filip Dobrovolny 
'''
def analyzeLegoList(row):
    color = decodeColor(row[0])
    sizex = int(row[2])
    sizey = int(row[4])
    howMany = int(row[6:])
    lego1 = lego(color, sizex, sizey)
    lego1.rotate()
    return lego1, howMany



def parseLegoList(path):
    newSoubor = parseFile(path)
    x = int(newSoubor[0][0])
    y = int(newSoubor[0][1])
    type = newSoubor[0][2]
    kostky = []
    for i in range(1, len(newSoubor)-1):
        kostky.append(analyzeLegoList(newSoubor[i]))
        
    tower1 = tower(x, y, type)
    return tower1, kostky

def buildTower(tower, kostky):
    done = False
    while not done:
        tower, kostky, done = buildTowerFloor(tower, kostky)


def findKostku(kostky, delka, mensi=False):
    for i in kostky:
        if i[1] == 0:
            del(i)
        if i[0].sizey == delka and mensi == False:
            return kostky.index(i)
        elif i[0].sizey < delka and mensi == True:
            return kostky.index(i)
    return None
        
    
def buildYRow(y, kostky, patro, index, last):
    i = 0
    count = 0
    while i < y:# spocitame bila mista k zapleneni
        if patro[index][i] == "":
            count += 1
        i += 1
    
        
    
def buildTowerFloor(tower, kostky):
    x = tower.x
    y = tower.y
    patro = []
    for i in range(0, x):
        temp = []
        for z in range(0, y):
            temp.append(" ")
        patro.append(temp)
    for i in range(0, x):
        if i == x - 1:
            buildYRow(y, kostky, patro, i, last=True)
        
        
def drawPatro(patro, tower):
    # Inicializuje pygame
    pygame.init()
    mainClock = pygame.time.Clock()
    BLACK = (  0,   0,   0)
    WHITE = (255, 255, 255)
    BLUE =  (  0,   0, 255)
    GREEN = (  0, 255,   0)
    RED =   (255,   0,   0)
    # Nastavi proporce okna
    WINDOWWIDTH = 1000
    WINDOWHEIGHT = 1000
    windowSurface = pygame.display.set_mode((WINDOWWIDTH, WINDOWHEIGHT))
    pygame.display.set_caption("Vez")
    # spusti herni smycku
    boxY = WINDOWWIDTH / tower.y
    boxX = WINDOWHEIGHT / tower.x
    while True:
        mainClock.tick(1)
        # projde pygame eventy
        for event in pygame.event.get():
            if event.type == QUIT:
                pygame.quit()
                break
            # zkontroluje pygame eventy
           #if event.type == KEYDOWN:
        windowSurface.fill((255, 255, 255))
        for i in range(1, tower.x):
            pygame.draw.line(windowSurface, BLACK, [boxX * i, 0], [boxX*i,1000], 1)
        for i in range(1, tower.y):
            pygame.draw.line(windowSurface, BLACK, [0,boxY * i], [1000,boxY*i], 1)
        for x in range(0, len(tower.plan[patro])-1):
            for y in range(0, len(tower.plan[patro][x])-1):
                tempLego = tower.plan[patro][x][y]
                if tempLego != "":
                    pygame.draw.rect(windowSurface, tempLego.color, [boxX * x, boxY * y, boxX * (x+1), boxY * (y+1)])
        pygame.display.flip()
        

print("Filip Dobrovolny (c) 2014 - soutezici c. 61")

temp, path, patro = parseParams()
if temp == 1:
    tower = parseBuildPlan(path)
    try:
        drawPatro(patro, tower)
    except:
        print("Nahled ukončen")
else:
    for i in listdir(path):
        print("Stavim vez z kostek:", path+"/"+i)
        tower, kostky = parseLegoList(path+"/"+i)
